/* A Bison parser, made by GNU Bison 2.3.  */

/* Skeleton interface for Bison's Yacc-like parsers in C

   Copyright (C) 1984, 1989, 1990, 2000, 2001, 2002, 2003, 2004, 2005, 2006
   Free Software Foundation, Inc.

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 51 Franklin Street, Fifth Floor,
   Boston, MA 02110-1301, USA.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* Tokens.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
   /* Put the tokens into the symbol table, so that GDB and other debuggers
      know about them.  */
   enum yytokentype {
     IDENT = 258,
     UIDENT = 259,
     FCONST = 260,
     SCONST = 261,
     USCONST = 262,
     BCONST = 263,
     XCONST = 264,
     Op = 265,
     ICONST = 266,
     PARAM = 267,
     TYPECAST = 268,
     DOT_DOT = 269,
     COLON_EQUALS = 270,
     EQUALS_GREATER = 271,
     LESS_EQUALS = 272,
     GREATER_EQUALS = 273,
     NOT_EQUALS = 274,
     SQL_COMMENT = 275,
     C_COMMENT = 276,
     ABORT_P = 277,
     ABSENT = 278,
     ABSOLUTE_P = 279,
     ACCESS = 280,
     ACTION = 281,
     ADD_P = 282,
     ADMIN = 283,
     AFTER = 284,
     AGGREGATE = 285,
     ALL = 286,
     ALSO = 287,
     ALTER = 288,
     ALWAYS = 289,
     ANALYSE = 290,
     ANALYZE = 291,
     AND = 292,
     ANY = 293,
     ARRAY = 294,
     AS = 295,
     ASC = 296,
     ASENSITIVE = 297,
     ASSERTION = 298,
     ASSIGNMENT = 299,
     ASYMMETRIC = 300,
     ATOMIC = 301,
     AT = 302,
     ATTACH = 303,
     ATTRIBUTE = 304,
     AUTHORIZATION = 305,
     BACKWARD = 306,
     BEFORE = 307,
     BEGIN_P = 308,
     BETWEEN = 309,
     BIGINT = 310,
     BINARY = 311,
     BIT = 312,
     BOOLEAN_P = 313,
     BOTH = 314,
     BREADTH = 315,
     BY = 316,
     CACHE = 317,
     CALL = 318,
     CALLED = 319,
     CASCADE = 320,
     CASCADED = 321,
     CASE = 322,
     CAST = 323,
     CATALOG_P = 324,
     CHAIN = 325,
     CHAR_P = 326,
     CHARACTER = 327,
     CHARACTERISTICS = 328,
     CHECK = 329,
     CHECKPOINT = 330,
     CLASS = 331,
     CLOSE = 332,
     CLUSTER = 333,
     COALESCE = 334,
     COLLATE = 335,
     COLLATION = 336,
     COLUMN = 337,
     COLUMNS = 338,
     COMMENT = 339,
     COMMENTS = 340,
     COMMIT = 341,
     COMMITTED = 342,
     COMPRESSION = 343,
     CONCURRENTLY = 344,
     CONDITIONAL = 345,
     CONFIGURATION = 346,
     CONFLICT = 347,
     CONNECTION = 348,
     CONSTRAINT = 349,
     CONSTRAINTS = 350,
     CONTENT_P = 351,
     CONTINUE_P = 352,
     CONVERSION_P = 353,
     COPY = 354,
     COST = 355,
     CREATE = 356,
     CROSS = 357,
     CSV = 358,
     CUBE = 359,
     CURRENT_P = 360,
     CURRENT_CATALOG = 361,
     CURRENT_DATE = 362,
     CURRENT_ROLE = 363,
     CURRENT_SCHEMA = 364,
     CURRENT_TIME = 365,
     CURRENT_TIMESTAMP = 366,
     CURRENT_USER = 367,
     CURSOR = 368,
     CYCLE = 369,
     DATA_P = 370,
     DATABASE = 371,
     DAY_P = 372,
     DEALLOCATE = 373,
     DEC = 374,
     DECIMAL_P = 375,
     DECLARE = 376,
     DEFAULT = 377,
     DEFAULTS = 378,
     DEFERRABLE = 379,
     DEFERRED = 380,
     DEFINER = 381,
     DELETE_P = 382,
     DELIMITER = 383,
     DELIMITERS = 384,
     DEPENDS = 385,
     DEPTH = 386,
     DESC = 387,
     DETACH = 388,
     DICTIONARY = 389,
     DISABLE_P = 390,
     DISCARD = 391,
     DISTINCT = 392,
     DO = 393,
     DOCUMENT_P = 394,
     DOMAIN_P = 395,
     DOUBLE_P = 396,
     DROP = 397,
     EACH = 398,
     ELSE = 399,
     EMPTY_P = 400,
     ENABLE_P = 401,
     ENCODING = 402,
     ENCRYPTED = 403,
     END_P = 404,
     ENFORCED = 405,
     ENUM_P = 406,
     ERROR_P = 407,
     ESCAPE = 408,
     EVENT = 409,
     EXCEPT = 410,
     EXCLUDE = 411,
     EXCLUDING = 412,
     EXCLUSIVE = 413,
     EXECUTE = 414,
     EXISTS = 415,
     EXPLAIN = 416,
     EXPRESSION = 417,
     EXTENSION = 418,
     EXTERNAL = 419,
     EXTRACT = 420,
     FALSE_P = 421,
     FAMILY = 422,
     FETCH = 423,
     FILTER = 424,
     FINALIZE = 425,
     FIRST_P = 426,
     FLOAT_P = 427,
     FOLLOWING = 428,
     FOR = 429,
     FORCE = 430,
     FOREIGN = 431,
     FORMAT = 432,
     FORWARD = 433,
     FREEZE = 434,
     FROM = 435,
     FULL = 436,
     FUNCTION = 437,
     FUNCTIONS = 438,
     GENERATED = 439,
     GLOBAL = 440,
     GRANT = 441,
     GRANTED = 442,
     GREATEST = 443,
     GROUP_P = 444,
     GROUPING = 445,
     GROUPS = 446,
     HANDLER = 447,
     HAVING = 448,
     HEADER_P = 449,
     HOLD = 450,
     HOUR_P = 451,
     IDENTITY_P = 452,
     IF_P = 453,
     ILIKE = 454,
     IMMEDIATE = 455,
     IMMUTABLE = 456,
     IMPLICIT_P = 457,
     IMPORT_P = 458,
     IN_P = 459,
     INCLUDE = 460,
     INCLUDING = 461,
     INCREMENT = 462,
     INDENT = 463,
     INDEX = 464,
     INDEXES = 465,
     INHERIT = 466,
     INHERITS = 467,
     INITIALLY = 468,
     INLINE_P = 469,
     INNER_P = 470,
     INOUT = 471,
     INPUT_P = 472,
     INSENSITIVE = 473,
     INSERT = 474,
     INSTEAD = 475,
     INT_P = 476,
     INTEGER = 477,
     INTERSECT = 478,
     INTERVAL = 479,
     INTO = 480,
     INVOKER = 481,
     IS = 482,
     ISNULL = 483,
     ISOLATION = 484,
     JOIN = 485,
     JSON = 486,
     JSON_ARRAY = 487,
     JSON_ARRAYAGG = 488,
     JSON_EXISTS = 489,
     JSON_OBJECT = 490,
     JSON_OBJECTAGG = 491,
     JSON_QUERY = 492,
     JSON_SCALAR = 493,
     JSON_SERIALIZE = 494,
     JSON_TABLE = 495,
     JSON_VALUE = 496,
     KEEP = 497,
     KEY = 498,
     KEYS = 499,
     LABEL = 500,
     LANGUAGE = 501,
     LARGE_P = 502,
     LAST_P = 503,
     LATERAL_P = 504,
     LEADING = 505,
     LEAKPROOF = 506,
     LEAST = 507,
     LEFT = 508,
     LEVEL = 509,
     LIKE = 510,
     LIMIT = 511,
     LISTEN = 512,
     LOAD = 513,
     LOCAL = 514,
     LOCALTIME = 515,
     LOCALTIMESTAMP = 516,
     LOCATION = 517,
     LOCK_P = 518,
     LOCKED = 519,
     LOGGED = 520,
     MAPPING = 521,
     MATCH = 522,
     MATCHED = 523,
     MATERIALIZED = 524,
     MAXVALUE = 525,
     MERGE = 526,
     MERGE_ACTION = 527,
     METHOD = 528,
     MINUTE_P = 529,
     MINVALUE = 530,
     MODE = 531,
     MONTH_P = 532,
     MOVE = 533,
     NAME_P = 534,
     NAMES = 535,
     NATIONAL = 536,
     NATURAL = 537,
     NCHAR = 538,
     NESTED = 539,
     NEW = 540,
     NEXT = 541,
     NFC = 542,
     NFD = 543,
     NFKC = 544,
     NFKD = 545,
     NO = 546,
     NONE = 547,
     NORMALIZE = 548,
     NORMALIZED = 549,
     NOT = 550,
     NOTHING = 551,
     NOTIFY = 552,
     NOTNULL = 553,
     NOWAIT = 554,
     NULL_P = 555,
     NULLIF = 556,
     NULLS_P = 557,
     NUMERIC = 558,
     OBJECT_P = 559,
     OBJECTS_P = 560,
     OF = 561,
     OFF = 562,
     OFFSET = 563,
     OIDS = 564,
     OLD = 565,
     OMIT = 566,
     ON = 567,
     ONLY = 568,
     OPERATOR = 569,
     OPTION = 570,
     OPTIONS = 571,
     OR = 572,
     ORDER = 573,
     ORDINALITY = 574,
     OTHERS = 575,
     OUT_P = 576,
     OUTER_P = 577,
     OVER = 578,
     OVERLAPS = 579,
     OVERLAY = 580,
     OVERRIDING = 581,
     OWNED = 582,
     OWNER = 583,
     PARALLEL = 584,
     PARAMETER = 585,
     PARSER = 586,
     PARTIAL = 587,
     PARTITION = 588,
     PASSING = 589,
     PASSWORD = 590,
     PATH = 591,
     PERIOD = 592,
     PLACING = 593,
     PLAN = 594,
     PLANS = 595,
     POLICY = 596,
     POSITION = 597,
     PRECEDING = 598,
     PRECISION = 599,
     PRESERVE = 600,
     PREPARE = 601,
     PREPARED = 602,
     PRIMARY = 603,
     PRIOR = 604,
     PRIVILEGES = 605,
     PROCEDURAL = 606,
     PROCEDURE = 607,
     PROCEDURES = 608,
     PROGRAM = 609,
     PUBLICATION = 610,
     QUOTE = 611,
     QUOTES = 612,
     RANGE = 613,
     READ = 614,
     REAL = 615,
     REASSIGN = 616,
     RECURSIVE = 617,
     REF_P = 618,
     REFERENCES = 619,
     REFERENCING = 620,
     REFRESH = 621,
     REINDEX = 622,
     RELATIVE_P = 623,
     RELEASE = 624,
     RENAME = 625,
     REPEATABLE = 626,
     REPLACE = 627,
     REPLICA = 628,
     RESET = 629,
     RESTART = 630,
     RESTRICT = 631,
     RETURN = 632,
     RETURNING = 633,
     RETURNS = 634,
     REVOKE = 635,
     RIGHT = 636,
     ROLE = 637,
     ROLLBACK = 638,
     ROLLUP = 639,
     ROUTINE = 640,
     ROUTINES = 641,
     ROW = 642,
     ROWS = 643,
     RULE = 644,
     SAVEPOINT = 645,
     SCALAR = 646,
     SCHEMA = 647,
     SCHEMAS = 648,
     SCROLL = 649,
     SEARCH = 650,
     SECOND_P = 651,
     SECURITY = 652,
     SELECT = 653,
     SEQUENCE = 654,
     SEQUENCES = 655,
     SERIALIZABLE = 656,
     SERVER = 657,
     SESSION = 658,
     SESSION_USER = 659,
     SET = 660,
     SETS = 661,
     SETOF = 662,
     SHARE = 663,
     SHOW = 664,
     SIMILAR = 665,
     SIMPLE = 666,
     SKIP = 667,
     SMALLINT = 668,
     SNAPSHOT = 669,
     SOME = 670,
     SOURCE = 671,
     SQL_P = 672,
     STABLE = 673,
     STANDALONE_P = 674,
     START = 675,
     STATEMENT = 676,
     STATISTICS = 677,
     STDIN = 678,
     STDOUT = 679,
     STORAGE = 680,
     STORED = 681,
     STRICT_P = 682,
     STRING_P = 683,
     STRIP_P = 684,
     SUBSCRIPTION = 685,
     SUBSTRING = 686,
     SUPPORT = 687,
     SYMMETRIC = 688,
     SYSID = 689,
     SYSTEM_P = 690,
     SYSTEM_USER = 691,
     TABLE = 692,
     TABLES = 693,
     TABLESAMPLE = 694,
     TABLESPACE = 695,
     TARGET = 696,
     TEMP = 697,
     TEMPLATE = 698,
     TEMPORARY = 699,
     TEXT_P = 700,
     THEN = 701,
     TIES = 702,
     TIME = 703,
     TIMESTAMP = 704,
     TO = 705,
     TRAILING = 706,
     TRANSACTION = 707,
     TRANSFORM = 708,
     TREAT = 709,
     TRIGGER = 710,
     TRIM = 711,
     TRUE_P = 712,
     TRUNCATE = 713,
     TRUSTED = 714,
     TYPE_P = 715,
     TYPES_P = 716,
     UESCAPE = 717,
     UNBOUNDED = 718,
     UNCONDITIONAL = 719,
     UNCOMMITTED = 720,
     UNENCRYPTED = 721,
     UNION = 722,
     UNIQUE = 723,
     UNKNOWN = 724,
     UNLISTEN = 725,
     UNLOGGED = 726,
     UNTIL = 727,
     UPDATE = 728,
     USER = 729,
     USING = 730,
     VACUUM = 731,
     VALID = 732,
     VALIDATE = 733,
     VALIDATOR = 734,
     VALUE_P = 735,
     VALUES = 736,
     VARCHAR = 737,
     VARIADIC = 738,
     VARYING = 739,
     VERBOSE = 740,
     VERSION_P = 741,
     VIEW = 742,
     VIEWS = 743,
     VIRTUAL = 744,
     VOLATILE = 745,
     WHEN = 746,
     WHERE = 747,
     WHITESPACE_P = 748,
     WINDOW = 749,
     WITH = 750,
     WITHIN = 751,
     WITHOUT = 752,
     WORK = 753,
     WRAPPER = 754,
     WRITE = 755,
     XML_P = 756,
     XMLATTRIBUTES = 757,
     XMLCONCAT = 758,
     XMLELEMENT = 759,
     XMLEXISTS = 760,
     XMLFOREST = 761,
     XMLNAMESPACES = 762,
     XMLPARSE = 763,
     XMLPI = 764,
     XMLROOT = 765,
     XMLSERIALIZE = 766,
     XMLTABLE = 767,
     YEAR_P = 768,
     YES_P = 769,
     ZONE = 770,
     FORMAT_LA = 771,
     NOT_LA = 772,
     NULLS_LA = 773,
     WITH_LA = 774,
     WITHOUT_LA = 775,
     MODE_TYPE_NAME = 776,
     MODE_PLPGSQL_EXPR = 777,
     MODE_PLPGSQL_ASSIGN1 = 778,
     MODE_PLPGSQL_ASSIGN2 = 779,
     MODE_PLPGSQL_ASSIGN3 = 780,
     UMINUS = 781
   };
#endif
/* Tokens.  */
#define IDENT 258
#define UIDENT 259
#define FCONST 260
#define SCONST 261
#define USCONST 262
#define BCONST 263
#define XCONST 264
#define Op 265
#define ICONST 266
#define PARAM 267
#define TYPECAST 268
#define DOT_DOT 269
#define COLON_EQUALS 270
#define EQUALS_GREATER 271
#define LESS_EQUALS 272
#define GREATER_EQUALS 273
#define NOT_EQUALS 274
#define SQL_COMMENT 275
#define C_COMMENT 276
#define ABORT_P 277
#define ABSENT 278
#define ABSOLUTE_P 279
#define ACCESS 280
#define ACTION 281
#define ADD_P 282
#define ADMIN 283
#define AFTER 284
#define AGGREGATE 285
#define ALL 286
#define ALSO 287
#define ALTER 288
#define ALWAYS 289
#define ANALYSE 290
#define ANALYZE 291
#define AND 292
#define ANY 293
#define ARRAY 294
#define AS 295
#define ASC 296
#define ASENSITIVE 297
#define ASSERTION 298
#define ASSIGNMENT 299
#define ASYMMETRIC 300
#define ATOMIC 301
#define AT 302
#define ATTACH 303
#define ATTRIBUTE 304
#define AUTHORIZATION 305
#define BACKWARD 306
#define BEFORE 307
#define BEGIN_P 308
#define BETWEEN 309
#define BIGINT 310
#define BINARY 311
#define BIT 312
#define BOOLEAN_P 313
#define BOTH 314
#define BREADTH 315
#define BY 316
#define CACHE 317
#define CALL 318
#define CALLED 319
#define CASCADE 320
#define CASCADED 321
#define CASE 322
#define CAST 323
#define CATALOG_P 324
#define CHAIN 325
#define CHAR_P 326
#define CHARACTER 327
#define CHARACTERISTICS 328
#define CHECK 329
#define CHECKPOINT 330
#define CLASS 331
#define CLOSE 332
#define CLUSTER 333
#define COALESCE 334
#define COLLATE 335
#define COLLATION 336
#define COLUMN 337
#define COLUMNS 338
#define COMMENT 339
#define COMMENTS 340
#define COMMIT 341
#define COMMITTED 342
#define COMPRESSION 343
#define CONCURRENTLY 344
#define CONDITIONAL 345
#define CONFIGURATION 346
#define CONFLICT 347
#define CONNECTION 348
#define CONSTRAINT 349
#define CONSTRAINTS 350
#define CONTENT_P 351
#define CONTINUE_P 352
#define CONVERSION_P 353
#define COPY 354
#define COST 355
#define CREATE 356
#define CROSS 357
#define CSV 358
#define CUBE 359
#define CURRENT_P 360
#define CURRENT_CATALOG 361
#define CURRENT_DATE 362
#define CURRENT_ROLE 363
#define CURRENT_SCHEMA 364
#define CURRENT_TIME 365
#define CURRENT_TIMESTAMP 366
#define CURRENT_USER 367
#define CURSOR 368
#define CYCLE 369
#define DATA_P 370
#define DATABASE 371
#define DAY_P 372
#define DEALLOCATE 373
#define DEC 374
#define DECIMAL_P 375
#define DECLARE 376
#define DEFAULT 377
#define DEFAULTS 378
#define DEFERRABLE 379
#define DEFERRED 380
#define DEFINER 381
#define DELETE_P 382
#define DELIMITER 383
#define DELIMITERS 384
#define DEPENDS 385
#define DEPTH 386
#define DESC 387
#define DETACH 388
#define DICTIONARY 389
#define DISABLE_P 390
#define DISCARD 391
#define DISTINCT 392
#define DO 393
#define DOCUMENT_P 394
#define DOMAIN_P 395
#define DOUBLE_P 396
#define DROP 397
#define EACH 398
#define ELSE 399
#define EMPTY_P 400
#define ENABLE_P 401
#define ENCODING 402
#define ENCRYPTED 403
#define END_P 404
#define ENFORCED 405
#define ENUM_P 406
#define ERROR_P 407
#define ESCAPE 408
#define EVENT 409
#define EXCEPT 410
#define EXCLUDE 411
#define EXCLUDING 412
#define EXCLUSIVE 413
#define EXECUTE 414
#define EXISTS 415
#define EXPLAIN 416
#define EXPRESSION 417
#define EXTENSION 418
#define EXTERNAL 419
#define EXTRACT 420
#define FALSE_P 421
#define FAMILY 422
#define FETCH 423
#define FILTER 424
#define FINALIZE 425
#define FIRST_P 426
#define FLOAT_P 427
#define FOLLOWING 428
#define FOR 429
#define FORCE 430
#define FOREIGN 431
#define FORMAT 432
#define FORWARD 433
#define FREEZE 434
#define FROM 435
#define FULL 436
#define FUNCTION 437
#define FUNCTIONS 438
#define GENERATED 439
#define GLOBAL 440
#define GRANT 441
#define GRANTED 442
#define GREATEST 443
#define GROUP_P 444
#define GROUPING 445
#define GROUPS 446
#define HANDLER 447
#define HAVING 448
#define HEADER_P 449
#define HOLD 450
#define HOUR_P 451
#define IDENTITY_P 452
#define IF_P 453
#define ILIKE 454
#define IMMEDIATE 455
#define IMMUTABLE 456
#define IMPLICIT_P 457
#define IMPORT_P 458
#define IN_P 459
#define INCLUDE 460
#define INCLUDING 461
#define INCREMENT 462
#define INDENT 463
#define INDEX 464
#define INDEXES 465
#define INHERIT 466
#define INHERITS 467
#define INITIALLY 468
#define INLINE_P 469
#define INNER_P 470
#define INOUT 471
#define INPUT_P 472
#define INSENSITIVE 473
#define INSERT 474
#define INSTEAD 475
#define INT_P 476
#define INTEGER 477
#define INTERSECT 478
#define INTERVAL 479
#define INTO 480
#define INVOKER 481
#define IS 482
#define ISNULL 483
#define ISOLATION 484
#define JOIN 485
#define JSON 486
#define JSON_ARRAY 487
#define JSON_ARRAYAGG 488
#define JSON_EXISTS 489
#define JSON_OBJECT 490
#define JSON_OBJECTAGG 491
#define JSON_QUERY 492
#define JSON_SCALAR 493
#define JSON_SERIALIZE 494
#define JSON_TABLE 495
#define JSON_VALUE 496
#define KEEP 497
#define KEY 498
#define KEYS 499
#define LABEL 500
#define LANGUAGE 501
#define LARGE_P 502
#define LAST_P 503
#define LATERAL_P 504
#define LEADING 505
#define LEAKPROOF 506
#define LEAST 507
#define LEFT 508
#define LEVEL 509
#define LIKE 510
#define LIMIT 511
#define LISTEN 512
#define LOAD 513
#define LOCAL 514
#define LOCALTIME 515
#define LOCALTIMESTAMP 516
#define LOCATION 517
#define LOCK_P 518
#define LOCKED 519
#define LOGGED 520
#define MAPPING 521
#define MATCH 522
#define MATCHED 523
#define MATERIALIZED 524
#define MAXVALUE 525
#define MERGE 526
#define MERGE_ACTION 527
#define METHOD 528
#define MINUTE_P 529
#define MINVALUE 530
#define MODE 531
#define MONTH_P 532
#define MOVE 533
#define NAME_P 534
#define NAMES 535
#define NATIONAL 536
#define NATURAL 537
#define NCHAR 538
#define NESTED 539
#define NEW 540
#define NEXT 541
#define NFC 542
#define NFD 543
#define NFKC 544
#define NFKD 545
#define NO 546
#define NONE 547
#define NORMALIZE 548
#define NORMALIZED 549
#define NOT 550
#define NOTHING 551
#define NOTIFY 552
#define NOTNULL 553
#define NOWAIT 554
#define NULL_P 555
#define NULLIF 556
#define NULLS_P 557
#define NUMERIC 558
#define OBJECT_P 559
#define OBJECTS_P 560
#define OF 561
#define OFF 562
#define OFFSET 563
#define OIDS 564
#define OLD 565
#define OMIT 566
#define ON 567
#define ONLY 568
#define OPERATOR 569
#define OPTION 570
#define OPTIONS 571
#define OR 572
#define ORDER 573
#define ORDINALITY 574
#define OTHERS 575
#define OUT_P 576
#define OUTER_P 577
#define OVER 578
#define OVERLAPS 579
#define OVERLAY 580
#define OVERRIDING 581
#define OWNED 582
#define OWNER 583
#define PARALLEL 584
#define PARAMETER 585
#define PARSER 586
#define PARTIAL 587
#define PARTITION 588
#define PASSING 589
#define PASSWORD 590
#define PATH 591
#define PERIOD 592
#define PLACING 593
#define PLAN 594
#define PLANS 595
#define POLICY 596
#define POSITION 597
#define PRECEDING 598
#define PRECISION 599
#define PRESERVE 600
#define PREPARE 601
#define PREPARED 602
#define PRIMARY 603
#define PRIOR 604
#define PRIVILEGES 605
#define PROCEDURAL 606
#define PROCEDURE 607
#define PROCEDURES 608
#define PROGRAM 609
#define PUBLICATION 610
#define QUOTE 611
#define QUOTES 612
#define RANGE 613
#define READ 614
#define REAL 615
#define REASSIGN 616
#define RECURSIVE 617
#define REF_P 618
#define REFERENCES 619
#define REFERENCING 620
#define REFRESH 621
#define REINDEX 622
#define RELATIVE_P 623
#define RELEASE 624
#define RENAME 625
#define REPEATABLE 626
#define REPLACE 627
#define REPLICA 628
#define RESET 629
#define RESTART 630
#define RESTRICT 631
#define RETURN 632
#define RETURNING 633
#define RETURNS 634
#define REVOKE 635
#define RIGHT 636
#define ROLE 637
#define ROLLBACK 638
#define ROLLUP 639
#define ROUTINE 640
#define ROUTINES 641
#define ROW 642
#define ROWS 643
#define RULE 644
#define SAVEPOINT 645
#define SCALAR 646
#define SCHEMA 647
#define SCHEMAS 648
#define SCROLL 649
#define SEARCH 650
#define SECOND_P 651
#define SECURITY 652
#define SELECT 653
#define SEQUENCE 654
#define SEQUENCES 655
#define SERIALIZABLE 656
#define SERVER 657
#define SESSION 658
#define SESSION_USER 659
#define SET 660
#define SETS 661
#define SETOF 662
#define SHARE 663
#define SHOW 664
#define SIMILAR 665
#define SIMPLE 666
#define SKIP 667
#define SMALLINT 668
#define SNAPSHOT 669
#define SOME 670
#define SOURCE 671
#define SQL_P 672
#define STABLE 673
#define STANDALONE_P 674
#define START 675
#define STATEMENT 676
#define STATISTICS 677
#define STDIN 678
#define STDOUT 679
#define STORAGE 680
#define STORED 681
#define STRICT_P 682
#define STRING_P 683
#define STRIP_P 684
#define SUBSCRIPTION 685
#define SUBSTRING 686
#define SUPPORT 687
#define SYMMETRIC 688
#define SYSID 689
#define SYSTEM_P 690
#define SYSTEM_USER 691
#define TABLE 692
#define TABLES 693
#define TABLESAMPLE 694
#define TABLESPACE 695
#define TARGET 696
#define TEMP 697
#define TEMPLATE 698
#define TEMPORARY 699
#define TEXT_P 700
#define THEN 701
#define TIES 702
#define TIME 703
#define TIMESTAMP 704
#define TO 705
#define TRAILING 706
#define TRANSACTION 707
#define TRANSFORM 708
#define TREAT 709
#define TRIGGER 710
#define TRIM 711
#define TRUE_P 712
#define TRUNCATE 713
#define TRUSTED 714
#define TYPE_P 715
#define TYPES_P 716
#define UESCAPE 717
#define UNBOUNDED 718
#define UNCONDITIONAL 719
#define UNCOMMITTED 720
#define UNENCRYPTED 721
#define UNION 722
#define UNIQUE 723
#define UNKNOWN 724
#define UNLISTEN 725
#define UNLOGGED 726
#define UNTIL 727
#define UPDATE 728
#define USER 729
#define USING 730
#define VACUUM 731
#define VALID 732
#define VALIDATE 733
#define VALIDATOR 734
#define VALUE_P 735
#define VALUES 736
#define VARCHAR 737
#define VARIADIC 738
#define VARYING 739
#define VERBOSE 740
#define VERSION_P 741
#define VIEW 742
#define VIEWS 743
#define VIRTUAL 744
#define VOLATILE 745
#define WHEN 746
#define WHERE 747
#define WHITESPACE_P 748
#define WINDOW 749
#define WITH 750
#define WITHIN 751
#define WITHOUT 752
#define WORK 753
#define WRAPPER 754
#define WRITE 755
#define XML_P 756
#define XMLATTRIBUTES 757
#define XMLCONCAT 758
#define XMLELEMENT 759
#define XMLEXISTS 760
#define XMLFOREST 761
#define XMLNAMESPACES 762
#define XMLPARSE 763
#define XMLPI 764
#define XMLROOT 765
#define XMLSERIALIZE 766
#define XMLTABLE 767
#define YEAR_P 768
#define YES_P 769
#define ZONE 770
#define FORMAT_LA 771
#define NOT_LA 772
#define NULLS_LA 773
#define WITH_LA 774
#define WITHOUT_LA 775
#define MODE_TYPE_NAME 776
#define MODE_PLPGSQL_EXPR 777
#define MODE_PLPGSQL_ASSIGN1 778
#define MODE_PLPGSQL_ASSIGN2 779
#define MODE_PLPGSQL_ASSIGN3 780
#define UMINUS 781




#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
typedef union YYSTYPE
#line 221 "gram.y"
{
	core_YYSTYPE core_yystype;
	/* these fields must match core_YYSTYPE: */
	int			ival;
	char	   *str;
	const char *keyword;

	char		chr;
	bool		boolean;
	JoinType	jtype;
	DropBehavior dbehavior;
	OnCommitAction oncommit;
	List	   *list;
	Node	   *node;
	ObjectType	objtype;
	TypeName   *typnam;
	FunctionParameter *fun_param;
	FunctionParameterMode fun_param_mode;
	ObjectWithArgs *objwithargs;
	DefElem	   *defelt;
	SortBy	   *sortby;
	WindowDef  *windef;
	JoinExpr   *jexpr;
	IndexElem  *ielem;
	StatsElem  *selem;
	Alias	   *alias;
	RangeVar   *range;
	IntoClause *into;
	WithClause *with;
	InferClause	*infer;
	OnConflictClause *onconflict;
	A_Indices  *aind;
	ResTarget  *target;
	struct PrivTarget *privtarget;
	AccessPriv *accesspriv;
	struct ImportQual *importqual;
	InsertStmt *istmt;
	VariableSetStmt *vsetstmt;
	PartitionElem *partelem;
	PartitionSpec *partspec;
	PartitionBoundSpec *partboundspec;
	RoleSpec   *rolespec;
	PublicationObjSpec *publicationobjectspec;
	struct SelectLimit *selectlimit;
	SetQuantifier setquantifier;
	struct GroupClause *groupclause;
	MergeMatchKind mergematch;
	MergeWhenClause *mergewhen;
	struct KeyActions *keyactions;
	struct KeyAction *keyaction;
	ReturningClause *retclause;
	ReturningOptionKind retoptionkind;
}
/* Line 1529 of yacc.c.  */
#line 1155 "gram.h"
	YYSTYPE;
# define yystype YYSTYPE /* obsolescent; will be withdrawn */
# define YYSTYPE_IS_DECLARED 1
# define YYSTYPE_IS_TRIVIAL 1
#endif



#if ! defined YYLTYPE && ! defined YYLTYPE_IS_DECLARED
typedef struct YYLTYPE
{
  int first_line;
  int first_column;
  int last_line;
  int last_column;
} YYLTYPE;
# define yyltype YYLTYPE /* obsolescent; will be withdrawn */
# define YYLTYPE_IS_DECLARED 1
# define YYLTYPE_IS_TRIVIAL 1
#endif


